import express from "express";
import cookieParser from "cookie-parser";
import ejs from "ejs";
import fetch from "node-fetch";

const app = express()
const port = process.argv[2];
const backend = process.argv[3];
const main = process.argv[4];

app.engine('.ejs', ejs.__express);
app.set('views', './views');
app.set('view engine', 'ejs');
app.use(cookieParser())
app.use(express.static('./public'));

app.get('/assets', async function (req, res) {
    res.render('assets', {
        ...(await getAuth(req)),
        data: await (fetch(backend + 'assets?tokenIn=' + req.cookies.token).then(x => x.json()))
    });
});

app.get('/embed/:id', async function (req, res) {
    res.render('embed', {
        ...(await getAuth(req)),
        data: await (fetch(backend + 'embed?id=' + req.params.id).then(x => x.json())),
        main
    });
});

app.get('/feed', async function (req, res) {
    res.render('feed', {
        ...(await getAuth(req))
    });
});

app.get('/studios', async function (req, res) {
    res.render('studios', {
        ...(await getAuth(req))
    });
});

app.get('/rules', async function (req, res) {
    res.render('rules', {
        ...(await getAuth(req))
    });
});


app.get('/studios/:studio', async function (req, res) {
    let data = await (fetch(backend + 'studio?studio=' + req.params.studio).then(x => x.json()));
    data.users = data.users
        .sort((a, b) => a.username.localeCompare(b.username))
        .filter(function (item, pos, ary) {
            return !pos || item.username != ary[pos - 1].username;
        });
    res.render('studio', {
        ...(await getAuth(req)),
        studio: req.params.studio,
        data
    });
});

app.get('/', async function (req, res) {
    res.render('main', {
        ...(await getAuth(req))
    });
});

app.get('/users/:user', async function (req, res) {
    res.render('user', {
        ...(await getAuth(req)),
        user: req.params.user,
        data: await (fetch(backend + 'user?user=' + req.params.user).then(x => x.json()))
    });
});

app.get('/register/', async function (req, res) {
    res.render('register', {
        ...(await getAuth(req))
    });
});

app.get('/login/', async function (req, res) {
    res.render('login', {
        ...(await getAuth(req))
    });
});

app.get('/upload/', async function (req, res) {
    res.render('upload', {
        ...(await getAuth(req))
    });
});

app.listen(port, () => { })

function getApi() {
    return {
        backend
    }
}

async function getAuth(req) {
    let cookie = req.cookies.token;

    let username = await fetch(backend + 'token?token=' + encodeURIComponent(cookie));
    username = await username.json();

    return {
        ...getApi(),
        'username': username.data,
        'ban': username.ban,
        'token': cookie
    }
}